@@ -1,5 +1,5 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991, 1992, 1993, 1994, 1995 Jamie Zawinski <jwz@mcom.com>
+ * Copyright (c) 1991, 1992, 1993, 1994, 1995 Jamie Zawinski <jwz@netscape.com>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -22,6 +22,8 @@
 
 #include "xdaliclock.h"
 
+extern int no_writable_cells;
+
 #ifdef __STDC__
 static struct frame *image_to_frame (XImage *image);
 static void flush_segment_buffer (Display *dpy, Drawable drawable, GC draw_gc);
@@ -41,6 +43,9 @@ static void fill_pig_cache (Display *dpy, Drawable drawable,
 static long fill_time_digits (void);
 static void wander (void);
 
+static void fill_borders (Display *dpy, Window window, GC gc,
+			  int x, int y, int width, int height);
+
 #endif
 
 #if defined (BUILTIN_FONT) || defined (BUILTIN_FONT_2)
@@ -82,7 +87,7 @@ static struct raw_number numbers [] = {
   { nine_bits, nine_width, nine_height },
   { colon_bits, colon_width, colon_height },
   { slash_bits, slash_width, slash_height },
-  0
+  { 0, }
 };
 
 #endif /* BUILTIN_FONT */
@@ -115,7 +120,7 @@ static struct raw_number numbers_2 [] = {
   { nine2_bits, nine2_width, nine2_height },
   { colon2_bits, colon2_width, colon2_height },
   { slash2_bits, slash2_width, slash2_height },
-  0
+  { 0, }
 };
 
 #endif /* BUILTIN_FONT_2 */
@@ -128,7 +133,7 @@ static int minutes_only;
 static enum { MMDDYY, DDMMYY, YYMMDD, YYDDMM, MMYYDD, DDYYMM } date_format;
 
 #ifdef SHAPE
-static int do_shape;
+int do_shape;
 #endif
 
 
@@ -552,11 +557,20 @@ one_step (current_frame, target_frame, tick)
 
 static char test_hack; /* gag */
 
+/* #define DRYRUN */
+
 static long
 fill_time_digits ()
 {
+#ifdef DRYRUN
+  static long clock = 799995420L;
+  struct tm *tm;
+  tm = localtime (&clock);
+  clock++;
+#else
   long clock = time ((time_t *) 0);
   struct tm *tm = localtime (&clock);
+#endif
   if (test_hack)
     {
       time_digits [0] = time_digits [1] = time_digits [2] =
@@ -767,7 +781,12 @@ initialize_window (dpy, window)
 #endif
 
   attribute_mask = (CWBackPixel | CWEventMask | CWDontPropagate);
-  attributes.background_pixel = bg_color.pixel;
+#ifdef SHAPE
+  if (do_shape)
+    attributes.background_pixel = fg_color.pixel;
+  else
+#endif
+    attributes.background_pixel = bg_color.pixel;
   attributes.event_mask = (xgwa.your_event_mask |
 			   KeyPressMask | ButtonPressMask |
 			   ButtonReleaseMask | StructureNotifyMask |
@@ -887,9 +906,11 @@ get_resources (dpy)
   if (buf) free (buf);
 
   buf = get_string_resource ("memory", "Memory");
-  if (!strcasecmp (buf, "high") ||
-      !strcasecmp (buf, "hi") ||
-      !strcasecmp (buf, "sleazy"))
+  if (buf == 0)
+    be_a_pig = 0;
+  else if (!strcasecmp (buf, "high") ||
+	   !strcasecmp (buf, "hi") ||
+	   !strcasecmp (buf, "sleazy"))
     be_a_pig = 2;
   else if (!strcasecmp (buf, "medium") ||
 	   !strcasecmp (buf, "med"))
@@ -907,8 +928,10 @@ get_resources (dpy)
   if (buf) free (buf);
 
   buf = get_string_resource ("datemode", "DateMode");
-  if (!strcasecmp (buf, "mmddyy") || !strcasecmp (buf, "mm/dd/yy") ||
-      !strcasecmp (buf, "mm-dd-yy"))
+  if (buf == 0)
+    date_format = MMDDYY;
+  else if (!strcasecmp (buf, "mmddyy") || !strcasecmp (buf, "mm/dd/yy") ||
+	   !strcasecmp (buf, "mm-dd-yy"))
     date_format = MMDDYY;
   else if (!strcasecmp (buf, "ddmmyy") || !strcasecmp (buf, "dd/mm/yy") ||
 	   !strcasecmp (buf, "dd-mm-yy"))
@@ -1068,7 +1091,8 @@ run_digital (dpy, window)
 	{
 	  int j;
 	  if (do_cycle)
-	    cycle_colors (dpy, xgwa.colormap, &fg_color, &bg_color);
+	    cycle_colors (dpy, xgwa.colormap, &fg_color, &bg_color,
+			  window, window_draw_gc, window_erase_gc);
 	  for (j = 0; j < ndigits; j++)
 	    {
 	      if (target_frames [j])
@@ -1166,8 +1190,24 @@ run_digital (dpy, window)
 			window_offset_x + (wander_x - (character_width / 2)),
 			window_offset_y + (wander_y - (character_height / 2)),
 			1);
+
+	  if (do_cycle &&
+#ifdef SHAPE
+	      !do_shape &&
+#endif
+	      no_writable_cells)
+	    fill_borders (dpy, window, window_erase_gc,
+			  window_offset_x + (wander_x - (character_width / 2)),
+			  window_offset_y + (wander_y - (character_height /2)),
+			  x_offsets [ndigits-1] + character_width,
+			  character_height);
+
 	  XFlush (dpy);
+#ifdef DRYRUN
+	  getc(stdin);
+#else
 	  TICK_SLEEP ();
+#endif
 	}
       memcpy (last_time_digits, time_digits, sizeof (time_digits));
 
@@ -1180,7 +1220,7 @@ run_digital (dpy, window)
 			   pixmap, ShapeSet);
       else
 #endif
-
+      {
 	if (wander_p && different_minute
 #ifdef SHAPE
 	    && !do_shape
@@ -1198,6 +1238,7 @@ run_digital (dpy, window)
 #endif
 	    wander ();
 	  }
+      }
 
       /* sends 28 bytes */
       XCopyPlane (dpy, pixmap, window, window_draw_gc, 0, 0,
@@ -1206,6 +1247,21 @@ run_digital (dpy, window)
 		  window_offset_y + (wander_y - (character_height / 2)),
 		  1);
 
+      if (do_cycle &&
+#ifdef SHAPE
+	  !do_shape &&
+#endif
+	  no_writable_cells)
+	fill_borders (dpy, window, window_erase_gc,
+		      window_offset_x + (wander_x - (character_width / 2)),
+		      window_offset_y + (wander_y - (character_height / 2)),
+		      x_offsets [ndigits-1] + character_width,
+		      character_height);
+
+#ifdef DRYRUN
+      event_loop (dpy, window);
+      getc(stdin);
+#else
     if (minutes_only)
       {
 	/* This is slightly sleazy: when in no-seconds mode, wake up
@@ -1230,7 +1286,8 @@ run_digital (dpy, window)
 	    else
 	      {
 		if (do_cycle)
-		  cycle_colors (dpy, xgwa.colormap, &fg_color, &bg_color);
+		  cycle_colors (dpy, xgwa.colormap, &fg_color, &bg_color,
+				window, window_draw_gc, window_erase_gc);
 		SEC_SLEEP ();
 	      }
 	  }
@@ -1252,10 +1309,50 @@ run_digital (dpy, window)
 	    event_loop (dpy, window);
 	  }
       }
+#endif
     }
 }
 
 
+static void
+fill_borders (dpy, window, gc, x, y, width, height)
+     Display *dpy;
+     Window window;
+     GC gc;
+     int x, y, width, height;
+{
+  XRectangle rects[4];
+  int i = 0;
+  if (y > 0)
+    {
+      rects[i].x = 0;
+      rects[i].y = 0;
+      rects[i].width = ~0;
+      rects[i].height = y;
+      i++;
+      rects[i].x = x;
+      rects[i].y = y + height;
+      rects[i].width = ~0;
+      rects[i].height = ~0;
+      i++;
+    }
+  if (x > 0)
+    {
+      rects[i].x = 0;
+      rects[i].y = y;
+      rects[i].width = x;
+      rects[i].height = ~0;
+      i++;
+      rects[i].x = x + width;
+      rects[i].y = y;
+      rects[i].width = ~0;
+      rects[i].height = height;
+      i++;
+    }
+  if (i > 0)
+    XFillRectangles (dpy, window, gc, rects, i);
+}
+
 static int
 event_loop (dpy, window)
      Display *dpy;
